/*
 * Copyright (c) 1981 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)curses.h	5.4 (Berkeley) 6/30/88
 */

# ifndef WINDOW

# include	<stdio.h>
# include	<termios.h>

# define	bool	char
# define	chtype	short
# define	reg	register

# ifndef TRUE
# define	TRUE	(1)
# define	FALSE	(0)
# endif
# define	ERR	(0)
# define	OK	(1)

# define	_ENDLINE	001
# define	_FULLWIN	002
# define	_SCROLLWIN	004
# define	_FLUSH		010
# define	_FULLLINE	020
# define	_IDLINE		040
# define	_STANDOUT	0400
# define	_NOCHANGE	-1

# define	_puts(s)	tputs(s, 0, _putchar)

typedef	struct termios	SGTTY;

/*
 * Capabilities from termcap
 */

extern bool     AM, BS, CA, DA, DB, EO, HC, HZ, IN, MI, MS, NC, NS, OS, UL,
		XB, XN, XT, XS, XX;
extern char	*AL, *BC, *BT, *CD, *CE, *CL, *CM, *CR, *CS, *DC, *DL,
		*DM, *DO, *ED, *EI, *K0, *K1, *K2, *K3, *K4, *K5, *K6,
		*K7, *K8, *K9, *HO, *IC, *IM, *IP, *KD, *KE, *KH, *KL,
		*KR, *KS, *KU, *LL, *MA, *ND, *NL, *RC, *SC, *SE, *SF,
		*SO, *SR, *TA, *TE, *TI, *UC, *UE, *UP, *US, *VB, *VS,
		*VE, *AL_PARM, *DL_PARM, *UP_PARM, *DOWN_PARM,
		*LEFT_PARM, *RIGHT_PARM;
extern char	PC;

/*
 * From the tty modes...
 */

extern bool	GT, NONL, UPPERCASE, normtty, _pfast;

struct _win_st {
	short		_cury, _curx;
	short		_maxy, _maxx;
	short		_begy, _begx;
	short		_flags;
	short		_ch_off;
	bool		_clear;
	bool		_leave;
	bool		_scroll;
	chtype		**_y;
	chtype		*_firstch;
	chtype		*_lastch;
	struct _win_st	*_nextp, *_orig;
};

# define	WINDOW	struct _win_st

extern bool	My_term, _echoit, _rawmode, _endwin;

extern char	*Def_term, ttytype[];

extern int	LINES, COLS, _tty_ch;

extern SGTTY	_tty, _res_tty;

extern WINDOW	*stdscr, *curscr;

/*
 *	Define VOID to stop lint from generating "null effect"
 * comments.
 */
# ifdef lints
int	__void__;
# define	VOID(x)	(__void__ = (int) (x))
# else
# define	VOID(x)	(x)
# endif

/*
 * psuedo functions for standard screen
 */
# define	addch(ch)	VOID(waddch(stdscr, ch))
# define	getch()		VOID(wgetch(stdscr))
# define	addbytes(da,co)	VOID(waddbytes(stdscr, da,co))
# define	addstr(str)	VOID(waddbytes(stdscr, str, strlen(str)))
# define	getstr(str)	VOID(wgetstr(stdscr, str))
# define	move(y, x)	VOID(wmove(stdscr, y, x))
# define	clear()		VOID(wclear(stdscr))
# define	erase()		VOID(werase(stdscr))
# define	clrtobot()	VOID(wclrtobot(stdscr))
# define	clrtoeol()	VOID(wclrtoeol(stdscr))
# define	insertln()	VOID(winsertln(stdscr))
# define	deleteln()	VOID(wdeleteln(stdscr))
# define	refresh()	VOID(wrefresh(stdscr))
# define	inch()		VOID(winch(stdscr))
# define	insch(c)	VOID(winsch(stdscr,c))
# define	delch()		VOID(wdelch(stdscr))
# define	standout()	VOID(wstandout(stdscr))
# define	standend()	VOID(wstandend(stdscr))

/*
 * mv functions
 */
#define	mvwaddch(win,y,x,ch)	VOID(wmove(win,y,x)==ERR?ERR:waddch(win,ch))
#define	mvwgetch(win,y,x)	VOID(wmove(win,y,x)==ERR?ERR:wgetch(win))
#define	mvwaddbytes(win,y,x,da,co) \
		VOID(wmove(win,y,x)==ERR?ERR:waddbytes(win,da,co))
#define	mvwaddstr(win,y,x,str) \
		VOID(wmove(win,y,x)==ERR?ERR:waddbytes(win,str,strlen(str)))
#define mvwgetstr(win,y,x,str)  VOID(wmove(win,y,x)==ERR?ERR:wgetstr(win,str))
#define	mvwinch(win,y,x)	VOID(wmove(win,y,x) == ERR ? ERR : winch(win))
#define	mvwdelch(win,y,x)	VOID(wmove(win,y,x) == ERR ? ERR : wdelch(win))
#define	mvwinsch(win,y,x,c)	VOID(wmove(win,y,x) == ERR ? ERR:winsch(win,c))
#define	mvaddch(y,x,ch)		mvwaddch(stdscr,y,x,ch)
#define	mvgetch(y,x)		mvwgetch(stdscr,y,x)
#define	mvaddbytes(y,x,da,co)	mvwaddbytes(stdscr,y,x,da,co)
#define	mvaddstr(y,x,str)	mvwaddstr(stdscr,y,x,str)
#define mvgetstr(y,x,str)       mvwgetstr(stdscr,y,x,str)
#define	mvinch(y,x)		mvwinch(stdscr,y,x)
#define	mvdelch(y,x)		mvwdelch(stdscr,y,x)
#define	mvinsch(y,x,c)		mvwinsch(stdscr,y,x,c)

/*
 * psuedo functions
 */

#define	clearok(win,bf)	 (win->_clear = bf)
#define	leaveok(win,bf)	 (win->_leave = bf)
#define	scrollok(win,bf) (win->_scroll = bf)
#define flushok(win,bf)	 (bf ? (win->_flags |= _FLUSH):(win->_flags &= ~_FLUSH))
#define	getyx(win,y,x)	 y = win->_cury, x = win->_curx
#define	winch(win)	 (win->_y[win->_cury][win->_curx] & 0377)

#define raw()	 (_tty.c_lflag &= ~(ICANON|ISIG),_pfast=_rawmode=TRUE, tcsetattr(_tty_ch,TCSANOW,&_tty))
#define noraw()	 (_tty.c_lflag |= ICANON,_rawmode=FALSE,_pfast=!(_tty.c_oflag&OPOST),tcsetattr(_tty_ch,TCSANOW,&_tty))
#define cbreak() (_tty.c_lflag &= ~(ICANON),_rawmode=TRUE,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define nocbreak() (_tty.c_lflag |= (ICANON|ISIG),_rawmode=FALSE,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define crmode() cbreak()	/* backwards compatability */
#define nocrmode() nocbreak()	/* backwards compatability */
#define echo()	 (_tty.c_lflag |= ECHO,_echoit = TRUE,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define noecho() (_tty.c_lflag &= ~ECHO,_echoit = FALSE,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define nl()	 (_tty.c_oflag |= OPOST,_pfast = _rawmode,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define nonl()	 (_tty.c_oflag &= ~OPOST,_pfast = TRUE,tcsetattr(_tty_ch,TCSANOW,&_tty))
#define	savetty() ((void) tcgetattr(_tty_ch, &_tty), _res_tty = _tty)
#define	resetty() (_tty = _res_tty, (void) tcsetattr(_tty_ch,TCSANOW,&_tty))

#define	erasechar()	(_tty.c_cc[VERASE])
#define	killchar()	(_tty.c_cc[VKILL])
#define baudrate()	(_tty.c_ospeed)

WINDOW	*initscr(void),
	*newwin(int, int, int, int),
	*subwin(WINDOW *, int, int, int, int);
void	box(WINDOW *, char, char),
	delwin(WINDOW *),
	endwin(void),
	mvcur(int, int, int, int),
	touchline(WINDOW *win, int y, int sx, int ex),
	touchwin(WINDOW *),
	wclrtobot(WINDOW *),
	wclrtoeol(WINDOW *),
	werase(WINDOW *);
int	mvprintw(int, int, char *, ...),
	mvwprintw(WINDOW *, int, int, char *, ...),
	printw(char *, ...),
	scroll(WINDOW *),
	waddbytes(WINDOW *, char *, int),
	waddch(WINDOW *, char),
	waddstr(WINDOW *, char *),
	wclear(WINDOW *),
	wdeleteln(WINDOW *win),
	wgetch(WINDOW *),
	wgetstr(WINDOW *, char *),
	winsch(WINDOW *, char),
	winsertln(WINDOW *win),
	wmove(WINDOW *, int, int),
	wprintw(WINDOW *, char *, ...),
	wrefresh(WINDOW *);
char	*getcap(char *),
	*longname(char *, char *)
	*wstandout(WINDOW *),
	*wstandend(WINDOW *);

/*
 * Used to be in unctrl.h.
 */
#define	unctrl(c)	_unctrl[(c) & 0177]
extern char *_unctrl[];

#if defined __386__
#  pragma library (curses3r termcap3r unix3r)
#elif defined __SMALL__
#  pragma library (cursess termcaps unixs)
#elif defined __COMPACT__
#  pragma library (cursesc termcapc unixc)
#elif defined __MEDIUM__
#  pragma library (cursesm termcapm unixm)
#elif defined __LARGE__
#  pragma library (cursesl termcapl unixl)
# endif

# endif
